@@ -4,10 +4,10 @@ $^W=1;
 
 my $prj     = 'latex-tds';
 my $file    = 'build.pl';
-my $version = cvs('$Revision: 1.172 $');
-my $date    = cvs('$Date: 2012/05/12 11:13:32 $');
+my $version = '1.180';
+my $date    = '2013-03-01';
 my $author  = 'Heiko Oberdiek';
-my $copyright = "Copyright 2006-2012 $author";
+my $copyright = "Copyright 2006-2013 $author";
 chomp(my $license = <<"END_LICENSE");
 % $copyright
 %
@@ -52,13 +52,14 @@ my @pkg_list = (
     'tds',
     'knuth',
     'etex',
+    'amsfonts',
 );
 
 my $zip_comment = <<'END_ZIP_COMMENT';
-*************************************************
-* This file is part of project 'latex-tds', see *
-* CTAN:macros/latex/contrib/latex-tds/README    *
-*************************************************
+***************************************************
+* This file is part of project 'latex-tds', see   *
+* CTAN:macros/latex/contrib/latex-tds/README.html *
+***************************************************
 END_ZIP_COMMENT
 
 my $error = "!!! Error:";
@@ -75,7 +76,20 @@ my $dir_patch = 'patch';
 my $dir_distrib = 'distrib';
 my $dir_texmf = 'texmf';
 chomp(my $cwd = `pwd`);
-my $dir_cache = "$cwd/cache";
+
+# cache directory
+use File::HomeDir;
+my $homedir = File::HomeDir->my_home;
+my $dir_cache;
+if ($homedir && -d "$homedir/.cache") {
+    $dir_cache = "$homedir/.cache/latex-tds";
+    -d $dir_cache or mkdir $dir_cache
+            or die "$error Cannot make directory `$dir_cache'!\n";
+    symlink $dir_cache, "$cwd/cache";
+}
+else {
+    $dir_cache = "$cwd/cache";
+}
 
 my $jar_pdfbox_rewrite = "$cwd/$dir_lib/pdfbox-rewrite.jar";
 my $jar_multivalent = "$cwd/$dir_lib/Multivalent20060102.jar";
@@ -88,9 +102,10 @@ my $file_ctan_distrib = "$cwd/$prj.zip";
 my $file_ziptimetree = get_perl_script('ziptimetree');
 my $file_adjust_checksum = get_perl_script('adjust_checksum');
 
+my $prg_asciidoc     = 'asciidoc';
+my $prg_bibtex       = 'bibtex';
 my $prg_checksum     = $file_adjust_checksum;
-my $prg_bibtex       = "bibtex";
-my $prg_chmod        = "chmod";
+my $prg_chmod        = 'chmod';
 my $prg_cp           = 'cp -p';
 my $prg_curl         = 'curl';
 my $prg_docstrip     = 'tex -shell-escape';
@@ -104,7 +119,8 @@ my $prg_ls           = "ls";
 my $prg_makeindex    = 'makeindex';
 my $prg_mkdir        = 'mkdir';
 my $prg_mv           = 'mv';
-my $prg_patch        = "patch";
+my $prg_patch        = 'patch';
+my $prg_lowriter     = 'lowriter';
 my $prg_lualatex     = 'lualatex';
 my $prg_lualatextds  = "lualatex -fmt=$cwd/$dir_build/lualatex-tds";
 my $prg_lualatextds2 = "lualatex -fmt=$cwd/$dir_build/lualatex-tds2";
@@ -118,8 +134,10 @@ my $prg_sed          = "sed";
 my $prg_sort         = "sort";
 my $prg_texhash      = "texhash";
 my $prg_unzip        = 'unzip';
+my $prg_w3m          = 'w3m';
 my $prg_weave        = 'weave';
 my $prg_wget         = 'wget';
+my $prg_wkhtmltopdf  = 'wkhtmltopdf'; # with patched qt
 my $prg_zip          = 'zip';
 my $prg_ziptimetree  = $file_ziptimetree;
 
@@ -253,8 +271,10 @@ if (@list_modules > 0) {
     download_ctan('babel',         'macros/latex/required');
     download_ctan('amslatex',      'macros/latex/required');
     download_ctan('amsrefs',       'macros/latex/contrib');
-    # download_ctan('amsrefs.tds',   'install/macros/latex/contrib');
+    download_ctan('amsrefs.tds',   'install/macros/latex/contrib');
     download_ctan('amscls.tds',    'install/macros/latex/required/amslatex');
+    download_ctan('math.tds',      'install/macros/latex/required/amslatex');
+    download_ctan('amsfonts.tds',  'install/fonts/');
     download_ctan('psnfss',        'macros/latex/required');
     download_ctan('tds',           '');
     download_ctan('texware',       'systems/knuth/dist');
@@ -370,20 +390,30 @@ section('Unpacking');
     map { unpack_ctan($_); } @required_list;
     if ($modules{'amslatex'}) {
         unpack_ams('amscls', "$dir_incoming_ctan/amscls.tds.zip");
-        #unpack_ams('amsrefs', "$dir_incoming_ctan/amsrefs.tds.zip");
-        unpack_ams('amsrefs', "$dir_incoming_ams/amsrefs.zip");
-        unpack_ams('amsmath', "$dir_incoming_ams/amsmath.zip");
-        # because of 00readme.txt and amsrefs.dtx
-        # 2012-05-08: 00readme.txt is renamed to README.
-        # 2012-05-08: amsrefs.dtx is fixed on CTAN.
-        unpacking('amslatex',
-                  "$dir_incoming_ctan/amsrefs.zip",
-                  "$dir_build/amslatex/ctan");
-        run("$prg_cp $dir_build/amslatex/ctan/amsrefs/README "
-                . "$dir_build/amslatex/texmf/source/latex/amsrefs/README");
+        unpack_ams('amsrefs', "$dir_incoming_ctan/amsrefs.tds.zip");
+        unpack_ams('amsmath', "$dir_incoming/ctan/math.tds.zip");
+        #unpack_ams('amsrefs', "$dir_incoming_ams/amsrefs.zip");
+        #unpack_ams('amsmath', "$dir_incoming_ams/amsmath.zip");
+        ## because of 00readme.txt and amsrefs.dtx
+        ## 2012-05-08: 00readme.txt is renamed to README.
+        ## 2012-05-08: amsrefs.dtx is fixed on CTAN.
+        ##unpacking('amslatex',
+        ##          "$dir_incoming_ctan/amsrefs.zip",
+        ##          "$dir_build/amslatex/ctan");
+        ##run("$prg_cp $dir_build/amslatex/ctan/amsrefs/README "
+        ##        . "$dir_build/amslatex/texmf/source/latex/amsrefs/README");
         # run("$prg_cp $dir_build/amslatex/ctan/amsrefs/amsrefs.dtx "
         #         . "$dir_build/amslatex/texmf/source/latex/amsrefs/amsrefs.dtx");
     }
+    if ($modules{'amsfonts'}) {
+        ensure_directory("$dir_build/amsfonts");
+        unpacking('amsfonts',
+                  "$dir_incoming_ctan/amsfonts.tds.zip",
+                  "$dir_build/amsfonts/texmf");
+        my $src = "$dir_build/amsfonts";
+        ensure_directory($src);
+        run("$prg_cp $dir_build/amsfonts/texmf/source/latex/amsfonts/* $src/");
+    }
     unpack_psnfss('lw35nfss');
     unpack_psnfss('freenfss');
     unpack_ctan('tds');
@@ -619,10 +649,16 @@ section('Patches after source install');
         chdir $cwd;
 
         patch('base/encguide.tex');
-        patch('base/lb2.err');
         patch('base/source2e.tex');
         patch('base/tlc2.err');
         patch('base/utf8ienc.dtx');
+        # lb2.err contains <CR><LF> line endings, a patch file
+        # created by diff in Linux would create mixed line endings
+        # causing trouble for subversion (Karl Berry).
+        # Therefore the percent character is inserted by sed.
+        run("$prg_sed -i -e '"
+           . 's/\\\\usepackage\[latin1]{inputenc}/%\\\\usepackage[latin1]{inputenc}/'
+           . "' $dir_build/base/lb2.err");
         run("$prg_recode latin1..utf8 $dir_build/base/lb2.err");
     }
 
@@ -645,6 +681,11 @@ section('Patches after source install');
         run("$prg_recode latin1..utf8 $dir_build/amslatex/amsrefs/changes.tex");
     }
 
+    if ($modules{'amsfonts'}) {
+        patch("amsfonts/amsfndoc.def");
+        patch("amsfonts/amsfndoc.tex");
+    }
+
     if ($modules{'babel'}) {
         patch("babel/hebrew.fdd");
     }
@@ -670,12 +711,12 @@ section('Docstrip');
     docstrip('tools',    'tools');
     docstrip('babel',    'babel');
 
-    # patch for amsthm.sty, part 1/2
-    if ($modules{'amslatex'}) {
-        chdir "$dir_build/amslatex/amscls";
-        run("$prg_docstrip ams-c1.ins");
-        chdir $cwd;
-    }
+    ## patch for amsthm.sty, part 1/2
+    #if ($modules{'amslatex'}) {
+    #    chdir "$dir_build/amslatex/amscls";
+    #    run("$prg_docstrip ams-c1.ins");
+    #    chdir $cwd;
+    #}
 }
 
 section('TDS cleanup');
@@ -695,17 +736,23 @@ section('TDS cleanup');
             *.template
             diffs-c.txt
         ];
+        run(sprintf "%s %s %s",
+            $prg_mv,
+            "$dir_tds/source/latex/amsmath/diffs-m.txt",
+            "$dir_tds/doc/latex/amsmath/"
+        );
         cleanup_tds 'source/latex/amsmath', qw[
-            diffs-m.txt
             amstex.sty
         ];
-        run("$prg_mv $dir_tds/source/latex/amsrefs/amsrefs.faq"
-            . " $dir_tds/doc/latex/amsrefs/");
-        cleanup_tds 'source/latex/amsrefs', qw[
-            cite-x*.tex
-            jr.bib
-            gktest.ltb
-        ];
+        {
+          my @list = map { glob("$dir_tds/source/latex/amsrefs/$_"); } qw[
+              amsrefs.faq
+          ];
+          # 2013-02-13: Classified as test files that remain in
+          # the source tree:
+          #   cite-x*.tex jr.bib
+          map { run("$prg_mv $_ $dir_tds/doc/latex/amsrefs/"); } @list;
+        }
         # CTAN:macros/latex/required/amslatex/other/*
         run("$prg_cp $dir_build/amslatex/other/amsbooka.sty"
             . " $dir_build/amslatex/texmf/tex/latex/amscls/amsbooka.sty");
@@ -853,14 +900,14 @@ section('Install tex doc');
         chdir $cwd;
     }
 
-    # patch for amsthm.sty, part 2/2
-    if ($modules{'amslatex'}) {
-        chdir "$dir_build/amslatex/amscls";
-        my $dest_dir = '../texmf/tex/latex/amscls';
-        ensure_directory($dest_dir);
-        install($dest_dir, 'amsthm.sty');
-        chdir $cwd;
-    }
+    ## patch for amsthm.sty, part 2/2
+    #if ($modules{'amslatex'}) {
+    #    chdir "$dir_build/amslatex/amscls";
+    #    my $dest_dir = '../texmf/tex/latex/amscls';
+    #    ensure_directory($dest_dir);
+    #    install($dest_dir, 'amsthm.sty');
+    #    chdir $cwd;
+    #}
 
 my $dummy = <<'END_DUMMY';
     if ($modules{'babel'}) {
@@ -1191,11 +1238,15 @@ if ($modules{'tools'}) {
     my ($mday, $month, $year) = splice @time, 3, 3;
     my $release_info = sprintf "%04d/%02d/%02d Tools overview (HO)",
         $year + 1900, $month + 1, $mday;
-    open(OUT, ">$texfile") or die "$error Cannot open `$texfile'!\n";
-    print OUT <<"END_HEADER";
-\\NeedsTeXFormat{LaTeX2e}
-\\ProvidesFile{tools.tex}%
-  [$release_info]
+    my $tools_old = '';
+    if (-f $texfile) {
+        open(IN, '<', $texfile) or die "$error Cannot open `$texfile'!\n";
+        while (<IN>) {
+            $tools_old .= $_ unless $. < 3 and /^\\(NeedsTeXFormat|ProvidesFile)/;
+        }
+        close(IN);
+    }
+    my $tools_new = <<"END_HEADER";
 $license
 \\documentclass{tools-overview}
 \\begin{document}
@@ -1230,16 +1281,32 @@ END_TRACE
         my $text = $db{$entry};
         $text =~ s/^\s*/  /mg;
         chomp $text;
-        print OUT <<"END_ENTRY";
+        $tools_new .= <<"END_ENTRY";
 \\entry{$entry}{%
 $text
 }%
 END_ENTRY
     }
-    print OUT <<'END_TRAILER';
+    $tools_new .=  <<'END_TRAILER';
 \end{document}
 END_TRAILER
-    close(OUT);
+    if ($tools_old eq $tools_new) {
+        info("tools.tex is already uptodate");
+    }
+    else {
+        open(OUT, ">$texfile") or die "$error Cannot open `$texfile'!\n";
+        print OUT <<"END_FILE";
+\\NeedsTeXFormat{LaTeX2e}
+\\ProvidesFile{tools.tex}[$release_info]
+END_FILE
+        print OUT $tools_new;
+        close(OUT);
+        info("tools.tex has changed");
+        foreach my $ext (qw[pdf log]) {
+            my $file_cache = "$dir_cache/tools/tools.$ext";
+            unlink $file_cache if -f $file_cache;
+        }
+    }
     cache 'tools', 'tools', sub {
         final_begin;
         run("$prg_pdflatextds tools.tex");
@@ -1306,25 +1373,25 @@ END_CODE
     chdir $cwd;
 }
 
+sub makeindex ($) {
+    my $doc = shift;
+    my $style;
+    $style = 'gind.ist' unless $doc eq 'amsldoc';
+    run_makeindex("$doc.idx", $style);
+}
+
+sub bibtex ($) {
+    my $doc = shift;
+
+    if ($doc =~ /^cite-x[bh]$/) {
+        run("$prg_bibtex $doc");
+    }
+}
+
 ### Generate documentation for amslatex
 if ($modules{'amslatex'}) {
     section('Documentation: amslatex');
 
-    sub makeindex ($) {
-        my $doc = shift;
-        my $style;
-        $style = 'gind.ist' unless $doc eq 'amsldoc';
-        run_makeindex("$doc.idx", $style);
-    }
-
-    sub bibtex ($) {
-        my $doc = shift;
-
-        if ($doc =~ /^cite-x[bh]$/) {
-            run("$prg_bibtex $doc");
-        }
-    }
-
     sub generate_doc ($$) {
         my $amspkg = shift;
         my $doc = shift;
@@ -1335,11 +1402,14 @@ if ($modules{'amslatex'}) {
                 or $doc eq 'thmtest'
                 or $doc eq 'cite-xs'
                 or $doc eq 'mathscinet';
-        $latextds = $prg_lualatextds2 if $doc eq 'amsldoc'
-                                      or $doc eq 'subeqn'
-                                      or $doc eq 'textcmds';
+        $latextds = $prg_lualatextds2
+                if $doc eq 'amsldoc'
+                or $doc eq 'subeqn'
+                or $doc eq 'textcmds';
         $latextds = $prg_pdflatex
                 if $doc eq 'thmtest';
+        $latextds = $prg_lualatex
+                if $doc eq 'amsrdoc';
 
         symlink $ams_drv, "$doc.drv";
         cache 'amslatex', $doc, sub {
@@ -1377,10 +1447,57 @@ if ($modules{'amslatex'}) {
     chdir "$dir_build/amslatex/amsrefs";
     symlink '../texmf', 'texmf';
     map { generate_doc 'amsrefs', $_; } qw[
-        cite-xa cite-xb cite-xh cite-xs
         amsrdoc changes
         amsrefs amsxport ifoption mathscinet pcatcode rkeyval textcmds
     ];
+    # 2013-02-13: Excluded as test files:
+    #   cite-xa cite-xb cite-xh cite-xs
+    chdir $cwd;
+}
+
+if ($modules{'amsfonts'}) {
+    section('Documentation: amsfonts');
+
+    sub generate_doc2 ($$) {
+        my $amspkg = shift;
+        my $doc = shift;
+        my $ams_drv = "$cwd/$dir_tex/ams.drv";
+        my $latextds = $prg_lualatextds;
+        $latextds = $prg_pdflatextds
+                if $doc eq 'eufrak'
+                or $doc eq 'euscript';
+
+        symlink $ams_drv, "$doc.drv";
+        cache 'amsfonts', $doc, sub {
+            run("$latextds -draftmode $doc.drv");
+            makeindex($doc);
+            bibtex($doc);
+            run("$latextds -draftmode $doc.drv");
+            makeindex($doc);
+            run("$latextds -draftmode $doc.drv");
+            makeindex($doc);
+            final_begin;
+            run("$latextds $doc.drv");
+            final_end;
+        };
+        install_gen_pdf('fonts', $amspkg, $doc);
+    }
+
+    chdir "$dir_build/amsfonts";
+    map {generate_doc2 'amsfonts', $_; } qw[
+        amsfonts amssymb cmmib57 eufrak euscript
+    ];
+    # plain: amsfndoc
+    {
+        my $doc = 'amsfndoc';
+        cache 'amslatex', $doc, sub {
+            run("$prg_pdftex -draftmode $doc.tex");
+            final_begin;
+            run("$prg_pdftex $doc.tex");
+            final_end;
+        };
+    }
+    install_gen_pdf('fonts', 'amsfonts', 'amsfndoc');
     chdir $cwd;
 }
 
@@ -1699,20 +1816,101 @@ if ($modules{'etex'}) {
 if ($modules{'source'}) {
     section('Module source');
 
-    my $dir_dest = "$dir_build/source/texmf/source/latex/latex-tds";
+    my $dir_build_source = "$dir_build/source";
+    my $dir_dest = "$dir_build_source/texmf/source/latex/latex-tds";
+    my $dir_doc = "$dir_build_source/texmf/doc/latex/latex-tds";
     my $dir_scripts = "$dir_build/source/texmf/scripts";
+    my $file_readme = 'README';
+    my $file_readme_html = 'README.html';
+    my $file_readme_pdf = 'README.pdf';
+    # my $file_readme_notoc_html = "$dir_build_source/README-notoc.html";
+    my $file_readme_w3m_txt = "$dir_build_source/README-w3m.txt";
+
+    ensure_directory($dir_build_source);
+
+    # generate README.html
+    run("$prg_asciidoc --backend=xhtml11 README.asciidoc");
+
+    # generate README.pdf
+    # run("$prg_asciidoc --out-file=$file_readme_notoc_html"
+    #     . " --backend=xhtml11"
+    #     . " -a disable-javascript"
+    #     . " -a toc!"
+    #     . " README.asciidoc"
+    # );
+    # run("$prg_lowriter --invisible --convert-to odt "
+    #         . "--outdir $dir_build_source $file_readme_notoc_html");
+    # run("$prg_lowriter --invisible --convert-to pdf "
+    #         . "--outdir $dir_build_source $dir_build_source/README-notoc.odt");
+    # run("$prg_cp -p $dir_build_source/README-notoc.pdf $file_readme_pdf");
+    run("$prg_wkhtmltopdf $file_readme_html $file_readme_pdf");
+
+    # run("$prg_cp -p README.asciidoc $dir_build_source/README");
+    run("$prg_w3m -dump README.html>$file_readme_w3m_txt");
+    my @lines_pre;
+    my @lines_toc;
+    my @lines_post;
+    my $state = 'pre';
+    open(IN, '<', $file_readme_w3m_txt)
+            or die "$error Cannot open `$file_readme_w3m_txt'!\n";
+    while (<IN>) {
+        if (/^README for project/) {
+            push @lines_pre, '=' x (length($_) - 1) . "\n";
+            push @lines_pre, $_;
+            push @lines_pre, '=' x (length($_) - 1) . "\n";
+            next;
+        }
+        if (/^Table of Contents/i) {
+            push @lines_pre, "\n";
+            push @lines_pre, "\n";
+            push @lines_pre, $_;
+            push @lines_pre, '=' x (length($_)-1) . "\n";
+            next;
+        }
+        if (/^(\d+)\.(\d+\.)? /) {
+            push @lines_post, "\n" unless $2;
+            push @lines_post, $_;
+            push @lines_post, ($2 ? '-' : '=') x length($_) . "\n";
+            my $prefix = ' ' x (3 * ($2 ? 2 : 1));
+            s/^(\d+)\.(\d+\.)? /($1<10 ? ' ' : '') . "$1.$2 "/e;
+            push @lines_toc, "$prefix$_";
+            next;
+        }
+        if (/^JavaScript must be enabled/) {
+            $state = 'post';
+            next;
+        }
+        if ($state eq 'pre') {
+            push @lines_pre, $_;
+        }
+        else {
+            push @lines_post, $_;
+        }
+    }
+    close(IN);
+    open(OUT, '>', $file_readme) or die "$error Cannot write `$file_readme'!\n";
+    print OUT @lines_pre, @lines_toc, @lines_post;
+    close(OUT);
 
     install $dir_dest, qw[
         build.pl
-        README
+        README.asciidoc
+        README-docinfo.html
     ];
+
     install "$dir_dest/tex", glob("$dir_tex/*.*");
     install "$dir_dest/patch", glob("$dir_patch/*.*");
     install "$dir_dest/lib", $file_ziptimetree;
     install "$dir_dest/lib", $file_adjust_checksum;
-    install "$dir_dest/license", "$dir_license/lppl.txt";
+    install "$dir_dest/license/latex-tds", "$dir_license/latex-tds/lppl.txt";
+    install "$dir_dest/license/adjust_checksum", "$dir_license/adjust_checksum/lppl.txt";
     install "$dir_dest/license/ziptimetree", "$dir_license/ziptimetree/lgpl.txt";
-    install $dir_distrib, 'README';
+    install $dir_distrib, $file_readme;
+    install $dir_distrib, $file_readme_html;
+    install $dir_distrib, $file_readme_pdf;
+    install $dir_doc, $file_readme;
+    install $dir_doc, $file_readme_html;
+    install $dir_doc, $file_readme_pdf;
 }
 
 ### Module latex-tds
@@ -1761,7 +1959,7 @@ section('Distrib');
 
     if ($opt_all) {
         chdir $dir_distrib;
-        my $cmd = "$prg_zip -0 $file_ctan_distrib README";
+        my $cmd = "$prg_zip -0 $file_ctan_distrib README README.html README.pdf";
         for my $pkg (sort @pkg_list) {
             $cmd .= " $pkg.tds.zip";
         }
